#include <ace/OS.h>
#include <tao/corba.h>

#include <iostream.h>
#include <fstream.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>

#include "Accessor.hh"

CORBA::ORB_var Accessor::getORB(int argc, char **argv) const
{
   CORBA::ORB_var orb= NULL;

   try
   {
      orb= CORBA::ORB_init( argc, argv, "TAO");
   }
   catch(const CORBA::Exception &e)
   {
      ACE_PRINT_EXCEPTION (e, "An exception has occured.!");
   }

   return orb;
}


CORBA::Boolean Accessor::getORB( CORBA::ORB_var pOrb) const
{
   // initialisations
   int argc= 0;
   char* * argv= NULL;
   CORBA::Boolean MYFALSE = 0;
   CORBA::Boolean MYTRUE = 1;

   try
   {
      pOrb= CORBA::ORB_init( argc, argv, "TAO");
   }
   catch(const CORBA::Exception &e)
   {
      ACE_PRINT_EXCEPTION (e, "An exception has occured.!");
      return MYFALSE;
   }

   if ( CORBA::is_nil( pOrb.in ()))
   {
      cout << "ERROR: NULL CORBA object" << endl;
      return MYFALSE;
   }

   return MYTRUE;
}

CORBA::Boolean Accessor::readIOR
(
const char* ior,
const CORBA::ORB_ptr &orb,
CORBA::Object_ptr *pObj,
const CORBA::Boolean bWrite,
const CORBA::Boolean bHome
) const
{
   CORBA::Boolean MYFALSE = 0;
   CORBA::Boolean MYTRUE = 1;

   if ( ACE_OS::strlen( ior)>255)
   {
      cerr << "ERROR: path to ior too long(255)" << endl;
      return MYFALSE;
   }
   char iorFile[255];
   if ( bHome)
   {
      const char *env= getenv( "PCS_DIR");
      if ( !env)
      {
         cerr << "ERROR: null environment symbol for OpenFusion home 'PCS_DIR'" << endl;
         return MYFALSE;
      }

      ACE_OS::strcpy( iorFile, env);
      ACE_OS::strcat( iorFile, ior);
   }
   else
   {
      ACE_OS::strcpy( iorFile, ior);
   }

   struct stat buf;
   if ( ( stat( iorFile, &buf)==0) && ( buf.st_size>0))
   {
      ifstream strm( iorFile);
      if( !strm)
      {
         cerr << "Unable to open file: " << iorFile;
         return MYFALSE;
      }
      else
      {
         char *refStr= new char[buf.st_size + 1];
         strm >> refStr;
         if ( bWrite)
         {
            cout << "IOR is : " << refStr << endl;
         }
         *pObj= orb->string_to_object( refStr);
         if ( CORBA::is_nil(*pObj))
         {
            cerr << "getIOR: Error - IOR is invalid" << endl;
            return MYFALSE;
         }
         delete [] refStr;
      }                                           //end else (!strm)
   }                                              //end if
   else
   {
      cerr << "Can't find file with service IOR!" << endl;
      return MYFALSE;
   }

   return MYTRUE;
}

void Accessor::writeIOR( const char* iorFile, const char* ior) const
{
   ofstream strm( iorFile);

   if (!strm)
   {
      cerr << "Unable to open file: " << iorFile;
      exit(1);
   }
   else
   {
      strm << ior;
      strm.close();
   }
}


CORBA::Boolean Accessor::getStringFromObject( CORBA::ORB_var pOrb, char **sResult, CORBA::Object_var pObj) const
{
   CORBA::Boolean MYFALSE = 0;
   CORBA::Boolean MYTRUE = 1;

   if ( CORBA::is_nil(pObj.in ()))
   {
      cerr << "ERROR: NULL Object" << endl;
      return MYFALSE;
   }

   try
   {
      *sResult= pOrb->object_to_string( pObj.in ());
   }
   catch (...)
   {
      cerr << "ERROR: Cannot stringify Object" << endl;
      return MYFALSE;
   }

   return MYTRUE;
}


CORBA::Boolean Accessor::getObjectFromString( const CORBA::ORB_var pOrb, const char *sIOR, CORBA::Object_var pObj) const
{
   CORBA::Boolean MYFALSE = 0;
   CORBA::Boolean MYTRUE = 1;

   CORBA::String_var str= ACE_OS::strdup( sIOR);
   try
   {
      pObj= pOrb->string_to_object(str.in ());
   }
   catch (...)
   {
      cerr << "ERROR: Cannot stringify Proxy Pull Supplier" << endl;
      return MYFALSE;
   }

   return MYTRUE;
}
