#include <ace/OS.h>
#include <tao/corba.h>
#include <tao/PortableServer/PortableServer.h>
#include <tao/PortableServer/POA.h>

#include <iostream.h>
#include <fstream.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>

#include "ObjectAdapter.hh"

void ObjectAdapter::initialiseRootPOA (CORBA::ORB_var &orb)
{
   CORBA::Object_var rootPOAObj = orb->resolve_initial_references ("RootPOA");
   m_rootPOA = PortableServer::POA::_narrow (rootPOAObj.in ());
   m_poaManager = m_rootPOA->the_POAManager ();
   m_poaManager->activate ();
}

ObjectAdapter::ObjectAdapter (CORBA::ORB_var& orb, 
                              const CORBA::Boolean bAgent) 
   : m_bAgent( bAgent), m_parentOA( NULL)
{
   initialiseRootPOA (orb);
   ACE_OS::strcpy( m_sPoaName, "thisPOA");
}

ObjectAdapter::ObjectAdapter (CORBA::ORB_var& orb, 
                              const char *sPoaName, 
                              const CORBA::Boolean bAgent) 
   : m_bAgent( bAgent)
{
   initialiseRootPOA (orb);
   ACE_OS::strcpy(m_sPoaName, sPoaName);
}

ObjectAdapter::ObjectAdapter (CORBA::ORB_var& orb, 
                              ObjectAdapter *adp, 
                              const char *sPoaName, 
                              const CORBA::Boolean bPersistent, 
                              const CORBA::Boolean bAgent)
   : m_bAgent( bAgent), m_parentOA( adp)
{
   initialiseRootPOA (orb);
   ACE_OS::strcpy(m_sPoaName, sPoaName);

   if ( !bPersistent)
      createTransient (orb);
   else
      createPersistent (orb);
}

void ObjectAdapter::createTransient (CORBA::ORB_var& orb, 
                                     PortableServer::ServantBase *pPusher)
{
   assert( pPusher);

   // create a transient object ref
   //
   CORBA::Boolean bPersistent = 0;
   create (orb, bPersistent);

   // activate the transient object ref
   //
   CORBA::String_var serverName= ACE_OS::strdup( "thisServer");
   addServantToAdapter (pPusher, serverName.in (), bPersistent);
}

void ObjectAdapter::createPersistent (CORBA::ORB_var& orb, 
                                      PortableServer::ServantBase *pPusher)
{
   assert( pPusher);

   // create a persistent object ref
   //
   CORBA::Boolean bPersistent = 1;
   create (orb, bPersistent);

   // activate the persistent object ref
   //
   CORBA::String_var serverName= ACE_OS::strdup( "thisServer");
   addServantToAdapter (pPusher, serverName.in (), bPersistent);
}

void ObjectAdapter::create( CORBA::ORB_var& orb, CORBA::Boolean bPersistent)
{
   // Create policies
   CORBA::PolicyList policies;
   if ( bPersistent)
   {
      policies.length (3);
      policies[(CORBA::ULong)0]= 
         m_rootPOA->create_lifespan_policy (PortableServer::PERSISTENT);
      policies[(CORBA::ULong)1]= 
         m_rootPOA->create_id_assignment_policy (PortableServer::USER_ID);
      policies[(CORBA::ULong)2]= 
         m_rootPOA->create_implicit_activation_policy 
                       (PortableServer::NO_IMPLICIT_ACTIVATION);
      cout << "INFO: created persistent policies" << endl;
   }
   else
   {
      policies.length (1);
      policies[(CORBA::ULong)0]= 
         m_rootPOA->create_lifespan_policy (PortableServer::TRANSIENT);
      cout << "INFO: created transient policy" << endl;
   }

   // Create a child POA with the right policies
   //
   CORBA::String_var poaName= ACE_OS::strdup( m_sPoaName);
   try
   {
      m_thisPOA = m_rootPOA->create_POA (poaName.in (), 
                                         m_poaManager.in (), 
                                         policies);
   }
   catch (const CORBA::Exception &e)
   {
      cout << e << endl;
   }
   catch (...)
   {
      cout << "ERROR: creating poa" << endl;
   }
}

void ObjectAdapter::createPersistent( CORBA::ORB_var& orb)
{
   CORBA::Boolean bPersistent = 1;
   create (orb, bPersistent);
}

void ObjectAdapter::createTransient( CORBA::ORB_var& orb)
{
   CORBA::Boolean bPersistent = 0;
   create (orb, bPersistent);
}

PortableServer::POA_var ObjectAdapter::getPOA ()
{
   return m_thisPOA;
}

ObjectAdapter* ObjectAdapter::getParent() const
{
   return m_parentOA;
}

char* ObjectAdapter::getPoaName ()
{
   return (char*)m_sPoaName;
}

void ObjectAdapter::addServantToAdapter (PortableServer::ServantBase *impl, 
                                         const char *sName,
                                         CORBA::Boolean bPersistent)
{
   CORBA::String_var servantName = ACE_OS::strdup (sName);

   if (bPersistent == 1)
   {
      // persistent object about to be activated, so create an id
      // and activate the object with it.
      //
      PortableServer::ObjectId_var serverId =
         PortableServer::string_to_ObjectId (servantName.in ());
      // Activate the servant with the ID on myPOA
      try
      {
         getPOA()->activate_object_with_id (serverId.in (), impl);
      }
      catch (const CORBA::Exception &e)
      {
         cerr << "ERROR: " << e << endl;
      }
   }
   else
   {
      // transient object about to be activated, so allow the system
      // to create an id for it.
      //
      try
      {
         PortableServer::ObjectId_var serverId = 
            getPOA()->activate_object (impl);
      }
      catch (const CORBA::Exception &e)
      {
         cerr << "ERROR: " << e << endl;
      }
   }
}

void ObjectAdapter::activateManager()
{
   m_thisPOA->the_POAManager()->activate();
}
