#include "Basic.hh"

#include <Common/Accessor.hh>
#include <iostream.h>
#include <stdlib.h>

#define LOG_EVENTS 4
#define NULL_OBJECT_REFERENCE -1
#define LOG_FULL -2
#define LOG_LOCKED -3
#define INVALID_GRAMMAR -4
#define INVALID_CONSTRAINT -5

Basic::Basic (CORBA::ORB_var orb)
{
   this->orb = orb;
}


DsLogAdmin::BasicLogFactory_var Basic::resolve (CORBA::ORB_var orb)
{
   CORBA::Object_var object = NULL;

   if ( CORBA::is_nil(orb.in ()))
   {
      cout << "ERROR: ORB not initialised." << endl;
      exit (NULL_OBJECT_REFERENCE);
   }

   try
   {
      object = orb->resolve_initial_references ("BasicLogFactory");
   }
   catch(...)
   {
      cerr << "Failed to resolve BasicLogFactory..." << endl;
      exit (NULL_OBJECT_REFERENCE);
   }

   DsLogAdmin::BasicLogFactory_var factory = DsLogAdmin::BasicLogFactory::_narrow (object.in ());

   this->basic_log_factory = factory;

   if ( CORBA::is_nil(factory.in ()))
   {
      cout << "ERROR: Cannot get Basic Log factory" << endl;
      exit (NULL_OBJECT_REFERENCE);
   }

   cout << "Resolved Basic Log Factory" << endl;

   return factory;
}


DsLogAdmin::BasicLog_var Basic::create (DsLogAdmin::BasicLogFactory_var factory)
{
   if ( CORBA::is_nil(factory.in ()))
   {
      cout << "ERROR: Cannot get Basic Log factory" << endl;
      exit (NULL_OBJECT_REFERENCE);
   }

   DsLogAdmin::LogFullActionType logFullAction = DsLogAdmin::wrap;
   CORBA::ULongLong max_size = 0; // 0 means "infinite"
   DsLogAdmin::LogId logId = 0;

   DsLogAdmin::BasicLog_var basic_log = factory->create (logFullAction, max_size, logId);

   cout << "Created log with id :" << logId << endl;

   return basic_log;
}

void Basic::write (DsLogAdmin::BasicLog_var log)
{
   DsLogAdmin::Anys data (LOG_EVENTS);
   data.length (LOG_EVENTS);

   double d = 42.424242;
   int n = 123456789;
   char a = 'a';

   data[0] <<= ACE_OS::strdup ("Hello world");
   data[1] <<= d;
   data[2] <<= n;
   data[3] <<= a;

   try
   {
      log->write_records (data);
   }
   catch (const DsLogAdmin::LogFull&)
   {
      cerr << "Log is full." << endl;
      exit (LOG_FULL);
   }
   catch(const DsLogAdmin::LogLocked&)
   {
      cerr << "Log is locked" << endl;
      exit (LOG_LOCKED);
   }
}

void Basic::query (DsLogAdmin::BasicLog_var log)
{
   char* grammar = "EXTENDED_TCL";
   char* constraint = "$.info == 'Hello world'";
   DsLogAdmin::Iterator_var iter;
   DsLogAdmin::RecordList_var records;


   try
   {
      cout << "Querying using constraint: " << constraint << endl;
      records = log->query (grammar, constraint, iter);

      if (records->length () == 1)
      {
         cout << "As expected:" << endl;
      }
      else
      {
         cout << "Expected 1:" << endl;
      }


      cout << "-------------------------" << endl;

      for (unsigned int i=0; i < records->length (); i++)
      {
         cout << "The ID of the LogRecord matching the constraint : "  << constraint << " is: "                        << (CORBA::ULong)(records[i].id)    << endl;
         cout << "-------------------------" << endl;
      }

      cout << endl;
   }
   catch (const DsLogAdmin::InvalidGrammar&)
   {
      cerr << "Invalid Grammar" << endl;
      exit (INVALID_GRAMMAR);
   }
   catch (const DsLogAdmin::InvalidConstraint&)
   {
      cerr << "Invalid Constraint" << endl;
      exit (INVALID_CONSTRAINT);
   }
}
