#include <ace/OS.h>
#include "Notify.hh"

#include <Common/Accessor.hh>
#include <Common/ObjectAdapter.hh>


int main (int argc, char **argv)
{
   try
   {
      Accessor utils;
      ObjectAdapter *adapter;
   
      ACE_Time_Value *tv = new ACE_Time_Value(ACE_Time_Value::zero);
   
      ACE_DECLARE_NEW_CORBA_ENV;
   
      CORBA::ORB_var pOrb;
   
      try
      {
         pOrb = utils.getORB (argc, argv);
         cout << "INFO: initialised orb" << endl;
      }
      catch(CORBA::Exception& e)
      {
         cout << "ERROR: Cannot initialise orb" << endl;
         cerr << e << endl;
   
         return 1;
      }
   
      pOrb->run (tv); // Do not block!
   
      Notify *notify = new Notify (pOrb);
   
      adapter = new ObjectAdapter (pOrb);
      adapter->createTransient (pOrb, notify);
   
      DsNotifyLogAdmin::NotifyLogFactory_var factory = notify->resolve (pOrb);
   
      DsNotifyLogAdmin::NotifyLog_var log = notify->create (factory);
   
      notify->setFilter (log);
   
      notify->connect (log);
   
      notify->generate ();
   
      notify->disconnect ();
   
      return 0;
   }
   catch(CORBA::Exception& sysEx)
   {
      cerr << sysEx << endl;
      return 1;
   }
}
