#include "AccountManager.hh"

AccountManagerImpl::AccountManagerImpl (CORBA::ORB_ptr orb)
{
   _orb = orb;
   util = new Util (_orb);
}

void AccountManagerImpl::createAccount (const char * name, CORBA::Float amount)
   ACE_THROW_SPEC ((CORBA::SystemException))
{
   try
   {
      account = getAccount (ACE_OS::strdup (name));

      cout << "The account with name:  " << name << " already exists." << endl;
   }
   catch (const Bank::AccountManager::NotFound&)
   {
      AccountImpl* acmeAccount = new AccountImpl (ACE_OS::strdup (name), amount);

      try
      {
         PortableServer::POA_var rootPOA = _default_POA ();

         PortableServer::POAManager_var rootManager = rootPOA->the_POAManager ();

         rootManager->activate ();

         rootPOA->activate_object (acmeAccount);

         cout << "Activated accountServant" << endl;

         CORBA::Object_var obj = acmeAccount->_this ();

         util->addAccount (name, obj.in ());

         cout << "Successfully created account with the following details..."<< endl;
         cout << "Account Holder's name: " << ACE_OS::strdup (name) << endl;
      }
      catch(const CORBA::Exception& e)
      {
         e._tao_print_exception ("Unexpected exception...");
      }
   }
   catch(const CORBA::SystemException& sysEx)
   {
      sysEx._tao_print_exception ("Unexpected SystemException:");
   }
}

Bank::Account_ptr AccountManagerImpl::getAccount (const char * name)
   ACE_THROW_SPEC ((CORBA::SystemException, Bank::AccountManager::NotFound))
{
   account = util->getAccount (ACE_OS::strdup (name));

   if(CORBA::is_nil (account))
   {
      throw Bank::AccountManager::NotFound ();
   }
   else
   {
      return Bank::Account::_duplicate (account);
   }
}
