#include "BankClient.hh"

#include <ace/OS.h>

#include <tao/corba.h>

#include "Naming/Bank/Util.hh"

#include <iostream.h>

int main (int argc, char **argv)
{
   try
   {
      CORBA::ORB_ptr orb = CORBA::ORB_init (argc, argv);

      Util* util = new Util (orb);

      Bank::AccountManager_var accMgr = util->getAccManagerObject ();

      assert (accMgr.in ());

      const char* names[10] =
      {
         "George",
         "Lenon",
         "Stephen",
         "Mary",
         "Gary",
         "Jonathan",
         "Luke",
         "Rachel",
         "Paul",
         "Steve"
      };

      for (CORBA::ULong i = 0; i < 10; i++)
      {
         try
         {
            accMgr->createAccount (names[i], 10000);
         }
         catch(const CORBA::SystemException& ex)
         {
            ex._tao_print_exception ("***SystemException.");
            return -4;
         }
      }

      accMgr->createAccount ("George", 10000);

      try
      {
         Bank::Account_var gvAccount = accMgr->getAccount ("George");

         cout << "Printing account details for:" << gvAccount->name () << endl;

         cout << "Current Balance for Account Holder: " << gvAccount->name () << " is: "
              << gvAccount->balance () << endl;

         gvAccount->deposit (21000);
         cout << "New Balance for Account Holder: " << gvAccount->name () << " is: "
              << gvAccount->balance () << endl;
         gvAccount->withdraw (9000);
         cout << "New Balance for Account Holder: " << gvAccount->name () << " is: "
              << gvAccount->balance () << endl;
         gvAccount->withdraw (3900);
         cout << "New Balance for AccountHolder : " << gvAccount->name () << " is: "
              << gvAccount->balance () << endl;

         Bank::Account_var rachelAccount = accMgr->getAccount ("Rachel");
         cout << "Current Balance for Account Holder: " << rachelAccount->name () <<
                  " is: " << rachelAccount->balance () << endl;

         orb->destroy ();
      }
      catch(const Bank::AccountManager::NotFound& ex)
      {
         ex._tao_print_exception ("Account Not Found");
         return -1;
      }
      catch(const CORBA::SystemException& ex)
      {
         ex._tao_print_exception ("SystemException:");
         return -2;
      }
   }
   catch(const CORBA::SystemException& ex)
   {
      ex._tao_print_exception ("SysException:");
      return -3;
   }
   catch(const Bank::Account::InsufficientFunds&)
   {
      cerr << "You cannot withdraw more money than you already have." << endl;
      return -1;
   }
   catch(const CORBA::Exception& ex)
   {
      ex._tao_print_exception ("Unexpected Exception:");
      return -3;
   }
   return 0;
}
