#include "BankManager.hh"

#include <ace/OS.h>

#include <tao/corba.h>
#include <tao/PortableServer/PortableServer.h>
#include <tao/PortableServer/POA.h>

#include "Naming/Bank/Util.hh"

#include <iostream.h>

int main (int argc, char** argv)
{
   try
   {
      CORBA::ORB_ptr orb = CORBA::ORB_init (argc, argv);

      AccountManagerImpl* accountManagerServant = new AccountManagerImpl (orb);
      Util* util = new Util (orb);

      assert (accountManagerServant);

      PortableServer::POA_var rootPOA = util->getPersistentRootPOA ();

      PortableServer::POAManager_var rootManager = rootPOA->the_POAManager ();

      rootManager->activate ();

      rootPOA->activate_object (accountManagerServant);

      CORBA::Object_var servantObject = rootPOA->servant_to_reference (accountManagerServant);

      util->buildNameGraph ();

      util->bindAccountManager (servantObject.in ());

      cout << "Waiting to receive incoming requests..." << endl;

      orb->run ();

      orb->destroy (); //release resources
   }
   catch(const CORBA::Exception& e)
   {
      e._tao_print_exception ("Caught exception");
      return -1;
   }
}
