#include <ace/OS.h>
#include <stdio.h>
#include <iostream.h>

#include "NamingClient.hh"

/*
 * @func This program resolves a reference to the Naming Service
 * and finds a video supplier object in a lookup for a given video
 * title.
 */
int main ( int argc, char **argv)
{
   try
   {
   // get a reference to Access Utilities
   
      Accessor utils;
      Naming names;
   
   // initialise orb
      CORBA::ORB_var pOrb;
      try
      {
         pOrb= utils.getORB(argc, argv);
         cout << "INFO: initialised orb" << endl;
      }
      catch(const CORBA::Exception& e)
      {
         cout << "ERROR: Cannot initialise orb" << endl;
         cerr << e << endl;
         return 1;
      }
   
   // get a reference to naming service
      CORBA::Object_var pObj= NULL;
   
      try
      {
         pObj = pOrb->resolve_initial_references ("NameService");
      }
      catch (...)
      {
         cerr << "Failed to resolve NameService" << endl;
         return -1;
      }
   
   // narrow to root context
      CosNaming::NamingContext_var inc;
      try
      {
         inc= CosNaming::NamingContext::_narrow( pObj.in ());
      }
      catch (...)
      {
         cout << "ERROR: getting root context" << endl;
         return 1;
      }
   
   // prepare  name context to service provider binding
      CosNaming::Name root;
      root.length( 1 );
      root[0].id= ACE_OS::strdup( "ServiceProvider");
      root[0].kind= ACE_OS::strdup( "server");
   
   
      CORBA::Object_ptr server;
   
   // resolve service provider (server) from naiming service
      names.resolveToObject (&root, inc.in (), &server);
   // narrow server to ServiceProvider implementation
      Server::ServiceProvider_var provider;
      try
      {
         provider= Server::ServiceProvider::_narrow( server);
      }
      catch (...)
      {
         cerr << "ERROR: cannot narrow to VideoServiceProvider" << endl;
         return 1;
      }
   
      if (CORBA::is_nil (provider.in ()))
      {
         cout << "ERROR: NULL VideoServiceProvider object" << endl;
         return 1;
      }
   
   // make a remote function invocation on service provider to get the companies
      Server::CompanyNameList *cList= provider->getCompanies();
      if ( cList)
      {
         cout << "\nAvailable companies:-" << endl;
      }
      for (int iCount=0; iCount < (int)cList->length(); iCount++)
      {
         cout << (const char *) ((*cList)[iCount]) << endl;
      }
   
   // resolve arbitary company from naming service
      CORBA::Object_var company;
      CosNaming::Name companyName;
      companyName.length( 3);
      companyName[0].id= ACE_OS::strdup( "Video Stream Supplier");
      companyName[1].id= ACE_OS::strdup( "Films");
      companyName[2].id= ACE_OS::strdup( "Videos' for Sale");
   
      CORBA::Object_ptr myObject;
      names.resolveToObject (&companyName, inc.in (), &myObject);
   
   // narrow to ServiceProvider
      Server::Company_var companyImpl;
      try
      {
         companyImpl= Server::Company::_narrow( myObject);
      }
      catch (const CORBA::Exception& e)
      {
         cerr << "ERROR: cannot narrow to Company" << e << endl;
         return 1;
      }
   
      if (CORBA::is_nil (companyImpl.in ()))
      {
         cout << "ERROR: NULL Company object" << endl;
         return 1;
      }
   
      CORBA::String_var sCat= ACE_OS::strdup( "Films");
      Server::OfferList *cOffer= NULL;
   
      try
      {
         CORBA::String_var ssCat= ACE_OS::strdup( "Films");
         cOffer= companyImpl->getOffers(ssCat.in ());
   
         if ( cOffer)
         {
            cout << "\nAvailable 'Films' feeds : " << endl;
            for (int iCount2=0; iCount2 < (int)cOffer->length(); iCount2++)
            {
               cout << (const char *) ((*cOffer)[iCount2]) << endl;
            }
         }
      }//end try
      catch (...)
      {
         if ( !cOffer)
            cout << "ERROR: null offers" << endl;
         else
            cout << "ERROR: problem in Company.getOffers" << endl;
         return 1;
      }
   
   
      cout << "Press Enter to exit..." << endl;
      int iIn= cin.get();
   
      return 0;
   }
   catch(CORBA::Exception& sysEx)
   {
      cerr << sysEx << endl;
      return 1;
   }
}
