#include <ace/OS.h>
#include <tao/corba.h>
#include <tao/PortableServer/PortableServer.h>
#include <iostream.h>

#include "NamingServer.hh"

int main ( int argc, char **argv)
{
   try
   {
   // Utilities
      Accessor utils;
   
   // initialise orb

      CORBA::ORB_var pOrb;
      try
      {
         pOrb= utils.getORB(argc, argv);
         cout << "INFO: initialised orb" << endl;
      }
      catch(CORBA::Exception& e)
      {
         cout << "ERROR: Cannot initialise orb" << endl;
         cerr << e << endl;
   
         return 1;
      }
   
   
   // get reference to naming service
      CORBA::Object_var pObj= NULL;
   
      try
      {
         pObj = pOrb->resolve_initial_references ("NameService");
      }
      catch (...)
      {
         cerr << "Failed to resolve the NameService" << endl;
         return 2;
      }
   
   // create the service provider singleton
      VideoSupplier::m_pServiceProvider = new VideoServiceProvider( pOrb);
   
      assert( VideoSupplier::m_pServiceProvider);
   
   // declare the initial context
      CosNaming::NamingContext_var inc;
      try
      {
         inc= CosNaming::NamingContext::_narrow(pObj.in ());
      }
      catch (...)
      {
         cout << "ERROR: getting root context" << endl;
         return 1;
      }
   
   // assign a type and id to the service provider object
      CosNaming::Name root;
      root.length( 1);
      root[0].id= ACE_OS::strdup( "ServiceProvider");
      root[0].kind= ACE_OS::strdup( "server");
   
   // bind the service provider into the root context from here
      try
      {
         inc->bind (root, VideoSupplier::m_pServiceProvider->getServant().in ());
      }
      catch(CosNaming::NamingContext::AlreadyBound e)
      {
   //    catch already bound and rebind on server re-start
         try
         {
            cout << "WARNING: rebinding ServiceProvider" << endl;
            inc->rebind
        (root, VideoSupplier::m_pServiceProvider->getServant().in ());
   
         }
         catch (...)
         {
            cout << "ERROR: rebinding" << endl;
            return 1;
         }
   
      }
   
   
   // create a few arbitary categories
      const char *cats[]= {   "Films",          //0
                        "Sporting Events",//1
                        "News Broadcasts",//2
                        "Soap Operas"     //3
                        };
   
   // create a few arbitary videos
      const char *vids[]= {   "Bridge On The River Kwai",//0
                        "Mutiny On The Bounty",    //1
                        "Lawrence Of Arabia",      //2
                        "Star Wars",               //3
                        "Great Expectations",      //4
                        "FA Cup Final",            //5
                        "BBC",                     //6
                        "CNN",                     //7
                        "Corenation Street",       //8
                        "Neighbours",              //9
                        "Wimbledon"                //10
                        };
   
   // create a few arbitary video suppliers
      NamingVideoSupplier videosRUs( "Videos' R us");
      videosRUs.addCategory( cats[0]);
      videosRUs.addCategory( cats[1]);
      videosRUs.addVideo( cats[0], vids[0]);
      videosRUs.addVideo( cats[0], vids[1]);
      videosRUs.addVideo( cats[1], vids[5]);
      videosRUs.addVideo( cats[1], vids[10]);
   // have the company bind itself into the naming service
      videosRUs.bind (inc.in ());
   
      NamingVideoSupplier videos4Sale( "Videos' for Sale");
      videos4Sale.addCategory( cats[0]);
      videos4Sale.addCategory( cats[1]);
      videos4Sale.addCategory( cats[2]);
      videos4Sale.addVideo( cats[0], vids[2]);
      videos4Sale.addVideo( cats[0], vids[3]);
      videos4Sale.addVideo( cats[0], vids[4]);
      videos4Sale.addVideo( cats[1], vids[10]);
      videos4Sale.addVideo( cats[2], vids[6]);
      videos4Sale.addVideo( cats[2], vids[7]);
   // have the company bind itself into the naming service
      videos4Sale.bind (inc.in ());
   
      NamingVideoSupplier videosOnDemand( "Videos' on Demand");
      videosOnDemand.addCategory( cats[2]);
      videosOnDemand.addCategory( cats[3]);
      videosOnDemand.addVideo( cats[2], vids[6]);
      videosOnDemand.addVideo( cats[3], vids[8]);
      videosOnDemand.addVideo( cats[3], vids[9]);
   // have the company bind itself into the naming service
      videosOnDemand.bind (inc.in ());
   
   // accept incoming requests on this server
      cout << "Ready..." << endl;
      pOrb->run();
   
   // clean up global pointer
      delete VideoSupplier::m_pServiceProvider;
   
      return 0;
   }
   catch(CORBA::Exception& sysEx)
   {
      cerr << sysEx << endl;
      return 1;
   }
}
