#include <ace/OS.h>
#include <iostream.h>
#include <string.h>
#include <tao/corba.h>

#include "Mail.hh"
#include "InputManager.hh"

/*
 * @func Constructs a InputManager object that initialises the command array
 *
 * @parm | sCreate | ref to create function
 *
 * @parm | sSend | ref to send function
 *
 * @parm | sReceive | ref to receive function
 *
 * @parm | sLogin | ref to login function
 *
 * @parm | sLogout | ref to logout function
 *
 */
InputManager::InputManager( CREATE sCreate,
                              SEND sSend,
                              RECEIVE sReceive,
                              LOGIN sLogin,
                              LOGOUT sLogout)
{
// set the default prompt
   setDefaultPrompt();
// prepare the help prompt
   ACE_OS::strcpy( m_sErrorString, "type create <user name>, send <user name> <message>, login <user name>, receive or exit");
   ACE_OS::strcpy( m_sAdmin, "ADMIN>");
   cout << "\n\n\n\n" << endl;
   cout << m_sErrorString << endl;
// initialise the command array
   m_command.size( VALID_COMMANDS);

   m_command[0].size( 3);
   m_command[1].size( 4);
   m_command[2].size( 2);
   m_command[3].size( 3);
   m_command[4].size( 2);
// populate the commands - create
   m_command[0][0]= (void*)"create";
   m_command[0][1]= (void*)"<name>";
   m_command[0][2]= (void*) sCreate;
// populate the commands - send
   m_command[1][0]= (void*)"send";
   m_command[1][1]= (void*)"<name>";
   m_command[1][2]= (void*)"<message>";
   m_command[1][3]= (void*) sSend;
// populate the commands - receive
   m_command[2][0]= (void*)"receive";
   m_command[2][1]= (void*) sReceive;
// populate the commands - login
   m_command[3][0]= (void*)"login";
   m_command[3][1]= (void*)"<name>";
   m_command[3][2]= (void*) sLogin;
// populate the commands - logout
   m_command[4][0]= (void*)"logout";
   m_command[4][1]= (void*) sLogout;
}

/*
 * @func destroy this object
 *
 */
InputManager::~InputManager(){}

/*
 * @func set the defualt prompt
 *
 */
void InputManager::setDefaultPrompt()
{
   ACE_OS::strcpy( m_cPrompt, getDefaultPrompt());
}

/*
 * @func get the defualt prompt
 *
 * @rdesc the defualt prompt
 */
char *InputManager::getDefaultPrompt()
{
   return "ADMIN>";
}

/*
 * @func get the current prompt
 *
 * @rdesc the current prompt
 */
char *InputManager::getPrompt()
{
   return m_cPrompt;
}

/*
 * @func is user logged on ?
 *
 * @rdesc true if logged on
 */
CORBA::Boolean InputManager::isLoggedOn()
{
   if ( ACE_OS::strcmp( getDefaultPrompt(), getPrompt()))
   {
      return 1;
   }
   return 0;
}

/*
 * @func get the command input
 *
 * @rdesc true if logged on
 */
CORBA::Boolean InputManager::getCommand()
{
   char cCommand[40];
   cout << m_cPrompt;
   cin.getline( cCommand, 40);

   return 1;
}

/*
 * @func sets the command prompt
 *
 * @parm | sPrompt| the command prompt
 */
void InputManager::setPrompt( const char *sPrompt)
{
   ACE_OS::strcpy( m_cPrompt, sPrompt);
}

/*
 * @func despatches the command input
 *
 * @rdesc returns false on error
 */
CORBA::Boolean InputManager::despatch() const
{
   CORBA::Boolean bExit= 0;
   while (1)
   {
      char sCommand[255];
      char sParse[255];
      cout << m_cPrompt;
      cin.getline( sCommand, 40);
#if defined DIGITAL_UNIX
   #if DIGITAL_UNIX < 0x500
      cin.ignore ();
   #endif
#endif
      ACE_OS::strcpy( sParse, sCommand);

      char *cToken= ACE_OS::strtok( (char *)sCommand, " ");
      CORBA::Boolean bError= 1;

      if ( cToken!=NULL)
      {
         while ( cToken!=NULL)
         {
            if ( !ACE_OS::strcmp( cToken, "exit"))
            {
               cout << "bye..." << endl;
               exit( 0);
            }

            for ( int iLoop=0; iLoop<VALID_COMMANDS; iLoop++)
            {
               if ( !ACE_OS::strcmp( cToken, (char *)m_command[iLoop][0]))
               {
                  bError= 1;
                  char *sParam1;
                  char *sParam2;
                  char *sRemainder;
                  switch( iLoop)
                  {
//                   create
                     case 0:
                        sParam1= ACE_OS::strtok( NULL, " ");
                        if (!sParam1) break;
                        ((CREATE)m_command[iLoop][2])( sParam1);
                        bError= 0;
                        break;
//                   send
                     case 1:
                        sParam1= ACE_OS::strtok( NULL, " ");
                        if (!sParam1) break;
                        sRemainder= ACE_OS::strtok( NULL, " ");
                        if ( !sRemainder) break;
                        sParam2= ACE_OS::strstr( sParse, sRemainder);
                        if ( !sParam2) break;
                        ((SEND)m_command[iLoop][3])( sParam1, sParam2);
                        bError= 0;
                        break;
//                   receive
                     case 2:
                        ((RECEIVE)m_command[iLoop][1])();
                        bError= 0;
                        break;
//                   login
                     case 3:
                        sParam1= ACE_OS::strtok( NULL, " ");
                        if (!sParam1) break;
                        ((LOGIN)m_command[iLoop][2])( sParam1);
                        bError= 0;
                        break;
//                   logout
                     case 4:
                        ((LOGOUT)m_command[iLoop][1])();
                        bError= 0;
                        break;
                     default:
                        bError= 1;
                  };
                  if ( bError)
                     cout << "ERROR: parsing command" << endl;
                  break;
               }
            }
            break;
         };//end while
      }//end if
      if ( bError)
         cout << "ERROR: invalid command\n" << m_sErrorString << endl;
   };//end while

   return bExit;
}

#if defined (ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION)
template class ACE_Array<void*>;
template class ACE_Array_Base<void*>;
template class ACE_Array_Iterator<void*>;
template class ACE_Array<VoidArray>;
template class ACE_Array_Base<VoidArray>;
template class ACE_Array_Iterator<VoidArray>;
#endif /* ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION */
