#include <ace/OS.h>
#include <iostream.h>
#include <tao/corba.h>

#include "PushSupplier.hh"
#include "PullConsumer.hh"
#include "Mail.hh"
#include "ChannelManager.hh"
#include "InputManager.hh"
#include "Common/Accessor.hh"


Accessor *pUtil= NULL;

CORBA::ORB_var pOrb;

PushSupplier *pSupplier= NULL;

PullConsumer *pConsumer= NULL;

ChannelManager *pManager= NULL;

InputManager *pCommand= NULL;

/*
 * @func command handler for create command
 *
 * @parm | sName | the user name
 *
 */
void create( char *sName)
{

   if ( !pManager)
   {
      cout << "ERROR: instantiating ChannelManager object"<< endl;
      return;
   }
   if ( !pUtil)
   {
      cout << "ERROR: instantiating Accessor object"<< endl;
      return;
   }

// set proxy name
   if ( !pManager->setUserName( sName))
   {
      return;
   }

// create proxy supplier
   if ( !pManager->getStructuredProxyPullSupplier())
   {
      return;
   }

// apply filter
   if ( !pManager->applyFilterToProxyPullSupplier())
   {
      return;
   }

// write out the IOR
   char *sIOR;
   if ( pManager->getStructuredProxyPullSupplierIOR( pOrb, &sIOR))
   {
      char sFileName[40];
      ACE_OS::strcpy( sFileName, pManager->getUserName());
      ACE_OS::strcat( sFileName, ".ior");
      cout << sFileName << "::::::" << sIOR << endl;
      pUtil->writeIOR( sFileName, sIOR);
   }
}

/*
 * @func command handler for send command
 *
 * @parm | sName | the user name
 *
 * @parm | sMessage | the message to send
 *
 */
void send( char *sName, char *sMessage)
{
   
   if ( !pManager)
   {
      cout << "ERROR: instantiating ChannelManager object"<< endl;
      return;
   }
   if ( !pCommand)
   {
      cout << "ERROR: instantiating InputManager object"<< endl;
      return;
   }

   if ( pCommand->isLoggedOn())
   {
      pManager->pushEvent(sName, sMessage);
   }
   else
   {
      cout << "ERROR: not logged on"<< endl;
   }
}

/*
 * @func command handler for login command
 *
 * @parm | sName | the user name
 *
 */
void login( char *sName)
{

   if ( !pManager)
   {
      cout << "ERROR: instantiating ChannelManager object"<< endl;
      return;
   }
   if ( !pCommand)
   {
      cout << "ERROR: instantiating InputManager object"<< endl;
      return;
   }
   if ( !pConsumer)
   {
      cout <<"ERROR: NULL consumer" << endl;
      return;
   }

   char sFileName[40];
   ACE_OS::strcpy( sFileName, sName);
   ACE_OS::strcat( sFileName, ".ior");
   CORBA::Object_ptr pObj= NULL;

   if ( pUtil->readIOR( sFileName, pOrb.in (), &pObj, 0, 0))
   {
      pManager->setUser( sName, pObj);
      pManager->connectConsumer( pConsumer);

      char sPrompt[40];
      ACE_OS::strcpy( sPrompt, sName);
      ACE_OS::strcat( sPrompt, ">");
      pCommand->setPrompt( sPrompt);
   }

}

/*
 * @func command handler for receive command
 *
 */
void receive()
{

   if ( !pManager)
   {
      cout << "ERROR: instantiating ChannelManager object"<< endl;
      return;
   }
   pManager->receiveEvent();
}

/*
 * @func command handler for logout command
 *
 */
void logout()
{

   if ( !pManager)
   {
      cout << "ERROR: instantiating ChannelManager object"<< endl;
      return;
   }

   if ( !pCommand)
   {
      cout <<"ERROR: NULL InputManager" << endl;
      return;
   }

   pManager->disconnectConsumer();
   pCommand->setDefaultPrompt();
}
/*
 * @func This program delegates to a ChannelManager object (singleton) that controls the Notification Service
 * structure, and to a InputManager object (singleton) that implements a simple command-line interpreter
 * The program also uses a utility object (singleton) exported to (DLL) openfusion.lib that provides IOR
 * read/write capabilities. This DLL also contains the OpenFusion IDL generated CPP object code.
 *
 */
int main ( int argc, char **argv)
{
   try
   {
   // command-to-function references
      CREATE pCreate= &create;
      SEND pSend= &send;
      RECEIVE pReceive= &receive;
      LOGIN pLogin= &login;
      LOGOUT pLogout= &logout;
   
      CORBA::Object_var obj= NULL;
   
      pUtil= new Accessor();
   
   // initialise orb
      try
      {
         pOrb= pUtil->getORB(argc, argv);
         cout << "INFO: initialised orb" << endl;
      }
      catch( CORBA::Exception& e)
      {
         cout << "ERROR: Cannot initialise orb" << endl;
         cerr << e << endl;
         return 1;
      }
   
   
      try
      {
         obj = pOrb->resolve_initial_references ("NotificationService");
      }
      catch (...)
      {
         cerr << "Failed to resolve initial references..!" << endl;
         return -1;
      }
   
   
   // instantiate manager on channel 0
      pManager= new ChannelManager( obj, 0);
   
   // create/connect supplier
      pSupplier= new PushSupplier();
      pManager->getStructuredProxyPushConsumer();
      pManager->connectSupplier( pSupplier);
   
   // create consumer
      pConsumer= new PullConsumer();
   
   
      pCommand= new InputManager( pCreate, pSend, pReceive, pLogin, pLogout);
      pCommand->despatch();
   
   // clean up globals
      delete pUtil;
      delete pSupplier;
      delete pConsumer;
      delete pManager;
      delete pCommand;
   
      return 0;
   }
   catch(CORBA::Exception& sysEx)
   {
      cerr << sysEx << endl;
      return 1;
   }
}
