#include <ace/OS.h>
#include <iostream.h>
#include <string.h>
#include <tao/corba.h>

#include "InputManager.hh"
#include "NewsInputManager.hh"

/*
 * @func Constructs a NewsInputManager object that initialises the command array
 *
 * @parm | sPublish | ref to publish function
 *
 * @parm | sBroadcast | ref to broadcast function
 *
 * @parm | sWithdraw | ref to withdraw function
 *
 */
NewsInputManager::NewsInputManager( PUBLISH sPublish, BROADCAST sBroadcast, WITHDRAW sWithdraw) :
                     InputManager( "commands: publish <weather> | <stock>, broadcast <weather> | <stock> ,withdraw <weather> | <stock>, exit")
{
// initialise the command array
   m_command.size( 3);

   m_command[0].size( 3);
   m_command[1].size( 3);
   m_command[2].size( 3);

// populate the commands - create
   m_command[0][0]= (void*)"publish";
   m_command[0][1]= (void*)"<domain>";
   m_command[0][2]= (void*) sPublish;

// populate the commands - broadcast
   m_command[1][0]= (void*)"broadcast";
   m_command[1][1]= (void*)"<domain>";
   m_command[1][2]= (void*) sBroadcast;

// populate the commands - withdraw
   m_command[2][0]= (void*)"withdraw";
   m_command[2][1]= (void*)"<domain>";
   m_command[2][2]= (void*) sWithdraw;
}

/*
 * @func Constructs a NewsInputManager object that initialises the command array
 *
 * @parm | sSubscribe| ref to subscribe function
 *
 */
NewsInputManager::NewsInputManager( SUBSCRIBE sSubscribe, LISTEN sListen) :
                     InputManager( "commands: subscribe <weather> | <stock> <priorty>, listen, exit")
{
// initialise the command array
   m_command.size( 2);

   m_command[0].size( 4);

// populate the commands - subscribe
   m_command[0][0]= ( void*)"subscribe";
   m_command[0][1]= (void*)"<domain>";
   m_command[0][2]= (void*)"<priority>";
   m_command[0][3]= (void*) sSubscribe;

   m_command[1].size( 2);

// populate the commands - subscribe
   m_command[1][0]= ( void*)"listen";
   m_command[1][1]= (void*) sListen;
}


/*
 * @func destroy this object
 *
 */
NewsInputManager::~NewsInputManager(){}

/*
 * @func despatches the command input
 *
 * @rdesc returns 0 on error
 */
CORBA::Boolean NewsInputManager::despatch()
{
   while (1)
   {
      char sCommand[255];
      char sParse[255];
      cout << getDefaultPrompt();
      cin.getline( sCommand, 255);
#if defined DIGITAL_UNIX
   #if DIGITAL_UNIX < 0x500
      cin.ignore ();
   #endif
#endif

      ACE_OS::strcpy( sParse, sCommand);

      char *cToken= ACE_OS::strtok( (char *)sCommand, " ");
      CORBA::Boolean bError= 1;

      if ( cToken!=NULL)
      {
         while ( cToken!=NULL)
         {
            if ( !ACE_OS::strcmp( cToken, "exit"))
            {
               //exit ( 0);
               return 1;
            }
            for ( int iLoop=0; iLoop<(int)m_command.size(); iLoop++)
            {
               if ( !ACE_OS::strcmp( cToken, (char *)m_command[iLoop][0]))
               {
                  bError= 1;
                  char *sParam1;
                  char *sParam2;
                  char *sRemainder;
                  int iPriority= 0;
                  switch( iLoop)
                  {
//                   publish
                     case 0:
                        sParam1= ACE_OS::strtok( NULL, " ");
                        if ( !sParam1) break;
                        if ( m_command.size()>2)//publish only
                        {
                           ((PUBLISH)m_command[iLoop][2])( sParam1);
                           bError= 0;
                           break;
                        }
                        else
                        {
//                         subscribe
                           sRemainder= ACE_OS::strtok( NULL, " ");
                           if ( !sRemainder) break;
                           sParam2= ACE_OS::strstr( sParse, sRemainder);
                           if ( !sParam2) break;
                           iPriority= atoi( sParam2);
                           ((SUBSCRIBE)m_command[iLoop][3])( sParam1, iPriority);
                           bError= 0;
                           break;
                        }
//                   broadcast
                     case 1:
                        if ( m_command.size()>2)//broadcast only
                        {
                           sParam1= ACE_OS::strtok( NULL, " ");
                           if ( !sParam1) break;
                           ((BROADCAST)m_command[iLoop][2])( sParam1);
                           bError= 0;
                           break;
                        }
                        else
                        {
//                         listen
                           ((LISTEN)m_command[iLoop][1])();
                           bError= 0;
                           break;
                        }
//                   withdraw
                     case 2:
                        sParam1= ACE_OS::strtok( NULL, " ");
                        if ( !sParam1) break;
                        ((WITHDRAW)m_command[iLoop][2])( sParam1);
                        bError= 0;
                        break;
                     default:
                        bError= 1;
                  };
                  if ( bError)
                     cout << "ERROR: parsing command" << endl;
                  break;
               }
            }//end for
            break;
         };//end while
      }//end if
      if ( bError)
         cout << "ERROR: invalid command\n" << getHelpPrompt() << endl;
   };//end while

   return 1;
}
