#include <ace/OS.h>
#include <iostream.h>
#include <tao/corba.h>

#include <CosNotifyCommS.h>
#include <CosNotifyChannelAdminS.h>
#include <CosNotificationS.h>

#include "PushConsumer.hh"
/*
 * @func Constructs a PushConsumer object
 *
 */
PushConsumer::PushConsumer( STOP sStop, START sStart) : m_stopFunc( sStop), m_startFunc( sStart)
{
   m_bWeather = 0;
   m_bStocks = 0;
}

/*
 * @func destroy this PushConsumer object
 *
 */
PushConsumer::~PushConsumer() {}

/*
 * @func called when disconnected from channel
 */
void PushConsumer::disconnect_structured_push_consumer() throw (CORBA::SystemException)
{
}

/*
 * @func called when event aggregate on channel changes
 *
 * @parm | eventTypeAdd | the given list of events added
 *
 * @parm | eventTypeRem | the given list of events removed
 *
 * @rdesc bool success
 */
void PushConsumer::offer_change( CosNotification::EventTypeSeq const &eventTypeAdd,
                                  CosNotification::EventTypeSeq const &eventTypeRem) throw (CosNotifyComm::InvalidEventType,CORBA::SystemException)
{


   CORBA::Boolean bFound= 0;
   for ( int iCountAdd= 0; iCountAdd< (CORBA::Long)eventTypeAdd.length(); iCountAdd++)
   {
      cout <<  eventTypeAdd[iCountAdd].domain_name << ": offer published" << endl;
      if ( m_bWeather)
         if ( !ACE_OS::strcmp( eventTypeAdd[iCountAdd].domain_name, "weather"))
            bFound= 1;

      if ( m_bStocks)
         if ( !ACE_OS::strcmp( eventTypeAdd[iCountAdd].domain_name, "stocks"))
            bFound= 1;
   }
   if ( bFound)
   {
      ((START)m_startFunc)();
      return;
   }

   for ( int iCountRem= 0; iCountRem < (int)eventTypeRem.length(); iCountRem++)
   {
      cout <<  eventTypeRem[iCountRem].domain_name << ": offer withdrawn" << endl;
      if ( m_bWeather)
         if ( !ACE_OS::strcmp( eventTypeAdd[iCountRem].domain_name, "weather"))
            bFound= 1;

      if ( m_bStocks)
         if ( !ACE_OS::strcmp( eventTypeAdd[iCountRem].domain_name, "stocks"))
            bFound= 1;
   }
   if ( bFound)
   {
      ((STOP)m_stopFunc)();
      return;
   }
}

/*
 * @func receive a structured event from the channel
 *
 * @parm | event | the given structured event
 */
void PushConsumer::push_structured_event( const CosNotification::StructuredEvent &event) throw (CosEventComm::Disconnected,CORBA::SystemException)
{
   try
   {
      CORBA::Any data= event.remainder_of_body;
      const char *sMessage;
      data >>= sMessage;
      cout << "INFO: received message: " << sMessage << endl;
   }
   catch (...)
   {
      cout << "ERROR: unpacking message" << endl;
   }
}

/*
 * @func sets the current event subscribed to
 *
 * @parm | iType | the given enumerated event type
 *
 */
void PushConsumer::setSubscribedEvent( const int iType)
{

   switch ( iType)
   {
      case 0:
         m_bWeather= 1;
         break;
      case 1:
         m_bStocks= 1;
         break;
   };
}

/*
 * @func sets the current event subscribed to
 *
 * @parm | iType | the given enumerated event type
 *
 */
CORBA::Boolean PushConsumer::getSubscribedType( const int iType)
{
   CORBA::Boolean bReturn= 0;

   switch ( iType)
   {
      case 0:
         bReturn=  m_bWeather;
         break;
      case 1:
         bReturn= m_bStocks;
         break;
   };

   return bReturn;
}
