#include <ace/OS.h>
#include <iostream.h>

#include "Notification/news/PushSupplier.hh"
#include "Notification/news/PullConsumer.hh"
#include "Notification/news/PushConsumer.hh"

#include "NewsPublisher.hh"
#include "Common/Accessor.hh"

#include "Common/ObjectAdapter.hh"
#include "Notification/news/ChannelManager.hh"
#include "Notification/news/NewsChannelManager.hh"

#include "Notification/news/InputManager.hh"
#include "Notification/news/NewsInputManager.hh"


   NewsChannelManager *pChannelManager= NULL;

   NewsInputManager *pCommand= NULL;

   PushSupplier *pSupplier= NULL;

#define STOCK_STRING "stock"
#define WEATHER_STRING "weather"

 char *weatherEvents[]=  {"Clear",
                           "Fine",
                           "Cloudy",
                           "Sunny",
                           "Showers",
                           "Snow",
                           "High Winds",
                           "Heavy Rain",
                           "Thunder and Lightening",
                           "Hurricane"};

 char *stockEvents[]=  {"Modest share dealings",
                           "Activity in IT shares",
                           "Manufacturing holdings up",
                           "Bank merger announced",
                           "Transport company bankrupt",
                           "Multi-national announce record profits",
                           "Retail company hostile take-over",
                           "Boom in oil shares",
                           "FTSE up dramatically",
                           "NASDAC crashes"};
/*
 * @func command handler for publish command
 *
 */
void publish( char* sDomain)
{
   if ( ACE_OS::strcmp( sDomain, WEATHER_STRING) && strcmp( sDomain, STOCK_STRING))
   {
      cout << "ERROR: invalid domain" << endl;
      return;
   }

   if ( !pChannelManager)
   {
      cout << "ERROR: no channel manager reference" << endl;
      return;
   }
   if ( !pCommand)
   {
      cout << "ERROR: no input manager reference" << endl;
      return;
   }
   int iType= 0;
   if ( !ACE_OS::strcmp( sDomain, STOCK_STRING))
      iType= 1;

   pChannelManager->addOffer( iType);
}

/*
 * @func command handler for broadcast command
 *
 */
void broadcast( char* sDomain)
{
   if ( ACE_OS::strcmp( sDomain, WEATHER_STRING) && strcmp( sDomain, STOCK_STRING))
   {
      cout << "ERROR: invalid domain" << endl;
      return;
   }

   if ( !pChannelManager)
   {
      cout << "ERROR: no channel manager reference" << endl;
      return;
   }
   if ( !pCommand)
   {
      cout << "ERROR: no input manager reference" << endl;
      return;
   }

   int iRand= ((float)(float)rand()/(float)RAND_MAX)*(float)9;
   int iIndex= abs( iRand);

   int iType;
   char sMessage[255];
   if ( !ACE_OS::strcmp( sDomain, STOCK_STRING))
   {
      iType= 1;
      ACE_OS::strcpy( sMessage, stockEvents[ iIndex]);
   }
   else
   {
      iType= 0;
      ACE_OS::strcpy( sMessage, weatherEvents[ iIndex]);
   }

   pChannelManager->pushEvent( sMessage, iIndex, iType);
   cout << "INFO: broadcast message: " << sMessage << endl;
}

/*
 * @func command handler for withdraw command
 *
 */
void withdraw( char* sDomain)
{
   if ( ACE_OS::strcmp( sDomain, WEATHER_STRING) && strcmp( sDomain, STOCK_STRING))
   {
      cout << "ERROR: invalid domain" << endl;
      return;
   }

   if ( !pChannelManager)
   {
      cout << "ERROR: no channel manager reference" << endl;
      return;
   }
   if ( !pCommand)
   {
      cout << "ERROR: no input manager reference" << endl;
      return;
   }
   int iType= 0;
   if ( !ACE_OS::strcmp( sDomain, STOCK_STRING))
      iType= 1;

   pChannelManager->removeOffer( iType);
}

int main ( int argc, char **argv)
{
   try
   {
   // the orb
      CORBA::ORB_var orb;
   // general purpose CORBA object
      CORBA::Object_ptr obj;
   // command-to-function references
      PUBLISH pPublish= &publish;
      BROADCAST pBroadcast= &broadcast;
      WITHDRAW pWithdraw= &withdraw;
   
      Accessor *utils= new Accessor();
   // initialise orb
      try
      {
         orb= utils->getORB(argc, argv);
         cout << "INFO: initialised orb" << endl;
      }
      catch(CORBA::Exception& e)
      {
         cout << "ERROR: Cannot initialise orb" << endl;
         cerr << e << endl;
         return 1;
      }
   
   
      try
      {
         obj = orb->resolve_initial_references ("NotificationService");
         ACE_TRY_CHECK;
      }
      catch (...)
      {
         cerr << "Failed to resolve NotificationService" << endl;
         return 1;
      }
   
   
      assert (obj);
   // create an event channel
      pChannelManager= new NewsChannelManager( obj, 0);
   
      if ( !pChannelManager)
      {
         cout << "ERROR: instantiating NewsChannelManager" << endl;
         return 1;
      }
   
      pCommand= NULL;
      pSupplier= new PushSupplier();
      if ( !pSupplier)
      {
         cout << "ERROR: instantiating PushSupplier" << endl;
         return 1;
      }
   
   // connect an implicitly activated servant (pSupplier) to the channel
      if ( pChannelManager->connectSupplier( pSupplier->_this(), 1))
      {
          pCommand= new NewsInputManager( pPublish, pBroadcast, pWithdraw);
          if ( pCommand)
             {
                pCommand->despatch();
             }
      }
   
      delete pChannelManager;
      delete pCommand;
      delete pSupplier;
      delete utils;
   
      return 0;
   }
   catch(CORBA::Exception& sysEx)
   {
      cerr << sysEx << endl;
      return 1;
   }
}
