#include <ace/OS.h>
#include <iostream.h>
#include <tao/corba.h>
#include <tao/PortableServer/POA.h>


#include "Notification/news/PushSupplier.hh"
#include "Notification/news/PushConsumer.hh"
#include "Notification/news/PullConsumer.hh"

#include "Notification/news/publisher/NewsPublisher.hh"
#include "Common/Accessor.hh"

#include "Common/ObjectAdapter.hh"
#include "Notification/news/ChannelManager.hh"
#include "Notification/news/NewsChannelManager.hh"

#include "Notification/news/InputManager.hh"
#include "Notification/news/NewsInputManager.hh"


   NewsChannelManager *pChannelManager= NULL;

   NewsInputManager *pCommand= NULL;

   PushConsumer *pConsumer= NULL;

   CORBA::ORB_var orb;

   ObjectAdapter *pAdapter= NULL;

   int g_iRestart= 1;

#define STOCK_STRING "stock"
#define WEATHER_STRING "weather"

/*
 * @func command to suspend the consumer connection
 *
 */
void stop()
{

   if ( !pChannelManager)
   {
      cout << "ERROR: no channel manager reference" << endl;
      return;
   }
// suspend the connection - remove the proxy
   pChannelManager->suspend();
}

/*
 * @func command to resume the consumer connection
 *
 */
void start()
{

   if ( !pChannelManager)
   {
      cout << "ERROR: no channel manager reference" << endl;
      return;
   }
// resume the connection
   pChannelManager->resume();
}

/*
 * @func command handler for subscribe command
 *
 */
void subscribe( char* sDomain, int iPriority)
{
   if ( ACE_OS::strcmp( sDomain, WEATHER_STRING) && strcmp( sDomain, STOCK_STRING))
   {
      cout << "ERROR: invalid domain" << endl;
      return;
   }

   if ( !pChannelManager)
   {
      cout << "ERROR: no channel manager reference" << endl;
      return;
   }
   if ( !pCommand)
   {
      cout << "ERROR: no input manager reference" << endl;
      return;
   }
   if ( !( iPriority>0 && iPriority<=10))
   {
      cout << "ERROR: 0 < priority <= 10" << endl;
      return;
   }
   if ( !pConsumer)
   {
      cout << "ERROR: no PushConsumer created" << endl;
      return;
   }

   int iType= 0;
   if ( !ACE_OS::strcmp( sDomain, STOCK_STRING))
      iType= 1;

   if ( !pChannelManager->getStructuredProxyPushSupplier())
      return;

   if ( pConsumer->getSubscribedType( iType))
   {
      cout << "INFO: already subscribed to this news item type" << endl;
      return;
   }
   pConsumer->setSubscribedEvent( iType);
   pChannelManager->applyFilter( iType, iPriority);
}

/*
 * @func command handler for subscribe command
 *
 */
void listen()
{
   if ( !pChannelManager)
   {
      cout << "ERROR: no channel manager reference" << endl;
      return;
   }
   if ( !pConsumer)
   {
      cout << "ERROR: no PushConsumer created" << endl;
      return;
   }
   if ( !pAdapter)
   {
      cout << "ERROR: no POA" << endl;
      return;
   }
      CosNotifyComm::StructuredPushConsumer_ptr pC= CosNotifyComm::StructuredPushConsumer::_narrow( pAdapter->getPOA()->servant_to_reference( pConsumer));
      if ( pC)
      {
//       Get event channel 0, if already exists use it - so any required properties can be set manually
         if ( pChannelManager->connectConsumer( pC, g_iRestart))
         {
            cout << "Ready..." << endl;
            orb->run();
            exit(0);
         }
      }
}

/*
 * @func main segment
 *
 */
int main ( int argc, char **argv)
{
   try
   {
   // general purpose CORBA object
      CORBA::Object_ptr obj;
   // command-to-function references
      SUBSCRIBE pSubscribe= &subscribe;
   // callback reference
      STOP pStop= &stop;
      START pStart= &start;
      LISTEN pListen= &listen;
   
      Accessor *utils= new Accessor();
   // initialise orb
      try
      {
         orb= utils->getORB(argc,argv);
         cout << "INFO: initialised orb " << endl;
      }
      catch (CORBA::Exception& e)
      {
         cout << "ERROR: Cannot initialise orb" << endl;
         cerr << e << endl;
         return 1;
      }
   
      try
      {
         obj = orb->resolve_initial_references ("NotificationService");
      }
      catch (...)
      {
         cerr << "Failed to resolve NotificationService" << endl;
         return 1;
      }
   
      assert (obj);

      // create an event channel
      pChannelManager= new NewsChannelManager( obj, 0);
   
      if ( !pChannelManager)
      {
         cout << "ERROR: instantiating NewsChannelManager" << endl;
         return 1;
      }
   
      pCommand= NULL;
      pConsumer= new PushConsumer( pStop, pStart);
      if ( !pConsumer)
      {
         cout << "ERROR: instantiating PushConsumer" << endl;
         return 1;
      }
   
      // the root OA
      ObjectAdapter *pRootAdp= NULL;
      pRootAdp= new ObjectAdapter (orb);
      if (!pRootAdp)
      {
         cout << "ERROR: instantiating root adapter" << endl;
         return 1;
      }
   
      // create an object adapter from root with persistent policy 
      // to manage consumer
      pAdapter= new ObjectAdapter (orb, pRootAdp, "ExampleSubscriberPoa", 1);
      if (!pAdapter)
      {
         cout << "ERROR: instantiating the adapter" << endl;
         return 1;
      }
   
      // add this servant
      pAdapter->addServantToAdapter (pConsumer,
                                     "ExampleSubscriberServant",
                                     1);
   
      // activate the poa manger
      pAdapter->activateManager ();

      pCommand= new NewsInputManager( pSubscribe, pListen);
      if ( pCommand)
      {
         pCommand->despatch();
      }
   
      delete pCommand;
      delete pChannelManager;
      delete pConsumer;
      delete pRootAdp;
      delete pAdapter;
   
      return 0;
   }
   catch(CORBA::Exception& sysEx)
   {
      cerr << sysEx << endl;
      return 1;
   }
}
