#include <ace/OS.h>

#include "Trading/Mortgage/Mortgage.hh"
#include "Trading/Mortgage/Util.hh"

#include <CosTradingC.h>
#include <CosTradingS.h>

#include <tao/corba.h>
#include <tao/PortableServer/PortableServer.h>
#include <tao/PortableServer/POA.h>

#include <iostream.h>

int main (int argc, char** argv)
{
   try
   {
      CORBA::ORB_ptr orb = CORBA::ORB_init (argc, argv);

      Util* util = new Util (orb);

      CORBA::Object_var obj = orb->resolve_initial_references ("TradingService");

      CosTrading::Lookup_var lookup = CosTrading::Lookup::_narrow (obj.in ());

      CosTrading::PolicySeq policies;
      CosTrading::Lookup::SpecifiedProps desiredProps;
      CosTrading::PropertyNameSeq_var property_seq = new CosTrading::PropertyNameSeq ();
      property_seq->length (1);
      property_seq[(CORBA::ULong)0] = ACE_OS::strdup ("financial_organisation");
      desiredProps.prop_names (property_seq.in ());

      CosTrading::OfferSeq_var offers;
      CosTrading::PolicyNameSeq_var policiesApplied;
      CosTrading::OfferIterator_var iterator;

      CORBA::String_var str= CORBA::string_dup("rate > 0.039");

      // execute query
      try
      {
         lookup->query( "IDL:Mortgage:1.0",
                        str,
                        "",
                        policies,
                        desiredProps,
                        4,
                        offers,
                        iterator,
                        policiesApplied);

      }
      catch( CORBA::Exception& e)
      {
         cout << "ERROR: querying: " << e << endl;
         return 1;
      }
      catch (...)
      {
         cout << "ERROR: querying" << endl;
         return 1;
      }

      Mortgage_var mortgage;
      cout << offers->length() << " Available Banks with " << str << endl;
      cout << "=================================" << endl;

      for ( CORBA::ULong ulCount= 0; ulCount< offers->length(); ulCount++)
      {

         const char *value = 0;

         try
         {
            cout << "Financial Organisation Name:";
            (*offers)[ulCount].properties[0].value >>= value;
            cout << value << endl;

            //Get object reference, narrow to correct type and invoke operation.
            mortgage = Mortgage::_narrow ((*offers)[ulCount].reference.in ());
            cout << "Calculating Mortgage payments." << endl;
            mortgage->calculateMortgage ((CORBA::Long)100000, (CORBA::Short)25);
         }
         catch (const CORBA::Exception& ex)
         {
            ex._tao_print_exception("Printing names failed");
            return 1;
         }
      }

      cout << "Press Enter to exit..." << endl;
      int iIn= cin.get();
   }
   catch(const CORBA::Exception& ex)
   {
      cerr << "Unexpected exception...." << endl;
      ex._tao_print_exception ("Unexpected exception:");
      return -1;
   }
   return 0;
}
