#include <ace/OS.h>

#include "Trading/Mortgage/Mortgage.hh"
#include "Trading/Mortgage/Util.hh"

#include <CosTradingC.h>
#include <CosTradingS.h>

#include <CosTradingReposC.h>
#include <CosTradingReposS.h>

#include <tao/corba.h>
#include <tao/PortableServer/PortableServer.h>
#include <tao/PortableServer/POA.h>

#include <iostream.h>

void export_mortgage_offer (CosTrading::Register_var register_if,
                            Mortgage_var mortgage,
                            char* name,
                            CORBA::Float rate);


int main (int argc, char** argv)
{
   try
   {
      CORBA::ORB_ptr orb = CORBA::ORB_init (argc, argv);

      Util* util = new Util (orb);

      CORBA::Object_var obj = orb->resolve_initial_references ("TradingService");

      CosTrading::Lookup_var lookup = CosTrading::Lookup::_narrow (obj.in ());

      CosTrading::TypeRepository_var type_repos_obj = lookup->type_repos ();

      CosTradingRepos::ServiceTypeRepository_var service_type_repos =
      CosTradingRepos::ServiceTypeRepository::_narrow (type_repos_obj.in ());

      CosTrading::Register_var register_if = lookup->register_if ();

      CosTradingRepos::ServiceTypeRepository::PropStructSeq_var properties = new CosTradingRepos::ServiceTypeRepository::PropStructSeq ();
      properties->length (2);

      properties[(CORBA::ULong)0].name = ACE_OS::strdup ("financial_organisation");
      properties[(CORBA::ULong)0].value_type = CORBA::_tc_string;
      properties[(CORBA::ULong)0].mode = CosTradingRepos::ServiceTypeRepository::PROP_MANDATORY_READONLY;

      properties[(CORBA::ULong)1].name = ACE_OS::strdup ("rate");
      properties[(CORBA::ULong)1].value_type = CORBA::_tc_float;
      properties[(CORBA::ULong)1].mode = CosTradingRepos::ServiceTypeRepository::PROP_NORMAL;

      CosTradingRepos::ServiceTypeRepository::ServiceTypeNameSeq_var super_types = new CosTradingRepos::ServiceTypeRepository::ServiceTypeNameSeq ();

      try
      {
         service_type_repos->add_type ("IDL:Mortgage:1.0",
                                       "IDL:Mortgage:1.0",
                                       properties,
                                       super_types);
      }
      catch (CosTradingRepos::ServiceTypeRepository::ServiceTypeExists &ex)
      {
         // Service type already exists. Assume it has been added from a 
         // previous run of this example. Remove the dead offers from it
         // because they are transient and inactive. Export the offers again.
         //
         cout << "Service type Mortgage already exists." << endl;
         cout << "Purging dead offers." << endl;

         CosTrading::Admin_var admin_if = lookup->admin_if ();
         CosTrading::OfferIdSeq_var idsSeq;
         CosTrading::OfferIdIterator_var idsIter;
         admin_if->list_offers (100, idsSeq, idsIter);
         for ( CORBA::ULong ulCount= 0; ulCount < idsSeq->length(); ulCount++)
         {
            register_if->withdraw (idsSeq [ulCount]);
         }
      }

      PortableServer::POA_var rootPOA = util->getPersistentRootPOA ();

      PortableServer::POAManager_var rootManager = rootPOA->the_POAManager ();

      rootManager->activate ();

      CORBA::Object_var mortgage_obj;
      Mortgage_var mortgage1,mortgage2,mortgage3,mortgage4;

      MortgageImpl* lloyds_mortgage = new MortgageImpl ("LloydsTSB", 5.9);
      MortgageImpl* halifax_mortgage = new MortgageImpl ("Halifax", 6.0);
      MortgageImpl* abbey_mortgage = new MortgageImpl ("ABBEY", 4.5);
      MortgageImpl* nationwide_mortgage = new MortgageImpl ("NationWide", 4.0);

      rootPOA->activate_object (lloyds_mortgage);
      rootPOA->activate_object (halifax_mortgage);
      rootPOA->activate_object (abbey_mortgage);
      rootPOA->activate_object (nationwide_mortgage);

      mortgage_obj = rootPOA->servant_to_reference (lloyds_mortgage);
      mortgage1 = Mortgage::_narrow (mortgage_obj.in ());

      export_mortgage_offer (register_if, mortgage1, mortgage1->name (), mortgage1->rate ());

      mortgage_obj = rootPOA->servant_to_reference (halifax_mortgage);
      mortgage2 = Mortgage::_narrow (mortgage_obj.in ());

      export_mortgage_offer (register_if, mortgage2, mortgage2->name (), mortgage2->rate ());

      mortgage_obj = rootPOA->servant_to_reference (abbey_mortgage);
      mortgage3 = Mortgage::_narrow (mortgage_obj.in ());

      export_mortgage_offer (register_if, mortgage3, mortgage3->name (), mortgage3->rate ());

      mortgage_obj = rootPOA->servant_to_reference (nationwide_mortgage);
      mortgage4 = Mortgage::_narrow (mortgage_obj.in ());

      export_mortgage_offer (register_if, mortgage4, mortgage4->name (), mortgage4->rate ());

      cout << "Successfully exported mortgage offers..." << endl;

      orb->run ();

      orb->destroy ();
   }
   catch(const CORBA::Exception& e)
   {
      e._tao_print_exception ("Exception:");
      return -1;
   }
   return 0;
}

void export_mortgage_offer (CosTrading::Register_var register_if,
                            Mortgage_var mortgage,
                            char* name,
                            CORBA::Float rate)
{
   CosTrading::PropertySeq_var properties = new CosTrading::PropertySeq ();

   properties->length (2);

   properties[(CORBA::ULong)0].name = ACE_OS::strdup ("financial_organisation");
   properties[(CORBA::ULong)0].value <<= name;
   properties[(CORBA::ULong)1].name = ACE_OS::strdup ("rate");
   properties[(CORBA::ULong)1].value <<= rate;

   register_if->_cxx_export (mortgage.in (), "IDL:Mortgage:1.0", properties);

   cout << "Successfully exported offer for " << name << " with APR " << rate << endl;
}
