#include <ace/OS.h>
#include <iostream.h>

#include <CosTradingC.h>
#include <CosTradingS.h>

#include <CosTradingReposC.h>
#include <CosTradingReposS.h>

#include "Trading.hh"

/*
 * @func Constructs a Trading object
 */
Trading::Trading() {}

/*
 * @func Destroy a Trading object
 *
 */
Trading::~Trading() {}

/*
 * @func gets a list of trader offers meeting given criteria
 *
 * @parm | pOrb | the given orb reference
 *
 * @parm | sConstraint | the given constaraint string
 *
 * @parm | sInterface | the given interface
 *
 * @parm | offers | offers received
 *
 * @parm | ulCount | the number of offers required
 *
 * @rdesc true on success
 */

CORBA::Boolean Trading::getTraderOffers( const CORBA::Object_ptr pObj, const char *sConstraint, const char *sInterface, CosTrading::OfferSeq_var offers, const unsigned long ulCount, CORBA::Boolean bFirst) const
{

// Query the trader
   CosTrading::Lookup_var lookup;
// get a reference to the lookup interface
   try
   {
      lookup= CosTrading::Lookup::_narrow( pObj);
   }
   catch (...)
   {
      cout << "ERROR: narrowing to lookup" << endl;
      return 0;
   }

// empty policy
   CosTrading::PolicySeq policies;
   CosTrading::Lookup::SpecifiedProps desiredProps;
   desiredProps._default();
   desiredProps._d( CosTrading::Lookup::none);

   CosTrading::PolicyNameSeq_var policiesApplied;
   CosTrading::OfferIterator_var iterator;

   try
   {
      lookup->query( sInterface,
                     sConstraint,
                     "",
                     policies,
                     desiredProps,
                     ulCount,
                     offers,
                     iterator,
                     policiesApplied);
   }
   catch (...)
   {
      cout << "ERROR: querying trader" << endl;
      return 0;
   }

   return 1;
}
