#include <ace/OS.h>
#include <tao/corba.h>
#include <tao/PortableServer/PortableServer.h>
#include <tao/PortableServer/POA.h>

#include <iostream.h>

#include "Common/Accessor.hh"
#include "Common/ObjectAdapter.hh"
#include "VideoCompany.hh"
#include "VideoServiceProvider.hh"
#include "VideoSupplier.hh"
#include "TradingVideoSupplier.hh"
/*
 * @func Constructs a TradingVideoSupplier
 *
 * @parm | sName | name of the video company
 */
TradingVideoSupplier::TradingVideoSupplier( CORBA::ORB_var orb, char *sName, const CORBA::Boolean bStore) : VideoSupplier( sName, bStore)
{
   VideoSupplier::m_pServiceProvider->store( orb, this, sName);
}

/*
 * @func destructs a CTradingVideoSupplier object
 *
 */
TradingVideoSupplier::~TradingVideoSupplier()
{
// delete offers;
}

/*
 * @func exports the managed video categories and titles
 *
 * @parm | lookup | reference to trader lookup interface
 *
 */
CORBA::Boolean TradingVideoSupplier::exportOffer( CORBA::ORB_var orb, CosTrading::Lookup_ptr lookup)
{
   StringArray *array;
   char **offer;
// arbitary video base cost
   float fBasePrice= 25.8f;

   if( !lookup || CORBA::is_nil( lookup))
   {
      cerr << "ERROR: lookup is nil." << endl;
      return 0;
   }

   CosTrading::Register_var register_obj= NULL;

   register_obj= lookup->register_if();
   if ( !register_obj.in () || CORBA::is_nil( register_obj.in ()))
   {
      cout << "ERROR: temp register is nil" << endl;
      return 0;
   }
   CORBA::Object_var obj = getServant();
   if ( !obj.in () || CORBA::is_nil( obj.in ()))
   {
      cout << "ERROR: bad CORBA::Object" << endl;
      return 0;
   }

   CosTrading::OfferId offer_id;
   CosTrading::PropertySeq properties;
   properties.length( 3);

   for (ASA_Iter itb = ASA_Iter (m_videos); itb.next (array) != 0; itb.advance ())
   {
      int iVideos= 0;
      for(StringArray::ITERATOR itb2 = StringArray::ITERATOR (*array); itb2.next (offer) != 0; itb2.advance ())
      {
//       randomise video cost
         float fInc= ( float(rand())/RAND_MAX)*10.0f;
         if ( ((int)fInc)>5)
            fInc*= -1.0f;

         fBasePrice+= fInc;
         if ( ((int)fBasePrice)<10)
            fBasePrice= 10.0f;

//       populate offer properties
         if ( iVideos>0)
         {
            cout << "INFO: cat: " << (*array)[0] << "| title: " << (*offer) << "| cost: " << fBasePrice << endl;

            CORBA::String_var sCat= ACE_OS::strdup( (*array)[0]);
            properties[0].name= ACE_OS::strdup( "type");
            properties[0].value <<= sCat;
//          video title
            CORBA::String_var sTitle= ACE_OS::strdup( (*offer));
            properties[1].name= ACE_OS::strdup( "title");
            properties[1].value <<= sTitle;
//          video cost
            properties[2].name= ACE_OS::strdup( "cost");
            properties[2].value <<= fBasePrice;
            try
            {
               cout << "About to export offers" << endl;
               offer_id= register_obj->_cxx_export( obj.in (), "IDL:video_supplier:1.0", properties);
               cout << "Successfully exported offers" << endl;
            }
            catch ( CORBA::SystemException& se)
            {
               cerr << "ERROR: exporting offer: " << se << endl;
               return 0;
            }
            catch (CosTrading::UnknownServiceType& se)
            {
               ACE_PRINT_EXCEPTION (se, "exportOffer");
               cerr << "ERROR: unknown type: " << " '" << se.type << "'" << endl;
               return 0;
            }
            catch (...)
            {
               cout << "ERROR: exporting offer" << endl;
               return 0;
            }

            cout << "INFO: exported: " << offer_id << endl;
         }//end if
         iVideos++;
      }//end for
   }//end for

   return 1;
}

/*
 * @func returns the CORBA reference of this object
 *
 * @rdesc the servant object reference
 */
CORBA::Object_var TradingVideoSupplier::getServant()
{
   return VideoSupplier::m_pServiceProvider->getVideoObjectAdapter()->getPOA()->servant_to_reference( this);
}
