#include <ace/OS.h>
#include <iostream.h>

#include <tao/corba.h>
#include <tao/PortableServer/PortableServer.h>
#include <tao/PortableServer/POA.h>

#include "VideoCompany.hh"
#include "VideoServiceProvider.hh"
#include "Common/ObjectAdapter.hh"

VideoServiceProvider::VideoServiceProvider( CORBA::ORB_var orb)
{
   CORBA::Boolean bPersistent = 0;
   m_adapter = new ObjectAdapter (orb, "ServiceProvider");
   m_adapter->createTransient (orb);
   m_adapter->addServantToAdapter (this, "provider", bPersistent);
}


VideoServiceProvider::~VideoServiceProvider()
{
   delete m_adapter;
   delete m_adapterVideos;
}

/*
 * @func IDL defined funtion that returns a list of companies managed by this service provider
 *
 * @rdesc void
 */
Server::CompanyNameList *VideoServiceProvider::getCompanies() throw (CORBA::SystemException)
{
   cout << "\nService Provider: managed companies:" << endl;

   Server::CompanyNameList *companies= new Server::CompanyNameList;
   companies->length( m_companies.size());
   VideoCompany **company;
   int iCount= 0;

   for(VideoCompanyArray::ITERATOR itb = VideoCompanyArray::ITERATOR (m_companies);
        itb.next (company) != 0; itb.advance ())
   {
      (*companies)[iCount]= ACE_OS::strdup( (*company)->getName());
      cout << (*company)->getName() << endl;
      iCount++;
   }
   return companies;
}

/*
 * @func maintains a store of VideoCompany objects
 *
 * @parm | pCompany | the given VideoCompany object
 *
 * @rdesc void
 */
void VideoServiceProvider::store( CORBA::ORB_var orb, VideoCompany *pCompany, char *sName)
{
   CORBA::Boolean bPersistent = 0;
   if ( m_companies.size() == 0)
   {
      m_adapterVideos = new ObjectAdapter (orb, "Videos");
      m_adapterVideos->createTransient (orb, pCompany);
   }
   else
   {
      m_adapterVideos->addServantToAdapter (pCompany, sName, bPersistent);
   }

   m_companies.size( m_companies.size()+1);
   m_companies[m_companies.size () - 1] = pCompany;
}


void VideoServiceProvider::activateManager()
{
   m_adapter->activateManager();
   m_adapterVideos->activateManager();
}


CORBA::Object_var VideoServiceProvider::getServant()
{
   return m_adapter->getPOA()->servant_to_reference( this);
}

void VideoServiceProvider::helloWorld() throw (CORBA::SystemException)
{
   cout << "hello World" << endl;
}

#if defined (ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION)
template class ACE_Array<VideoCompany*>;
template class ACE_Array_Base<VideoCompany*>;
template class ACE_Array_Iterator<VideoCompany*>;
#endif /* ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION */
