#include <ace/OS.h>
#include <iostream.h>
#include <tao/corba.h>
#include <tao/PortableServer/PortableServer.h>

#include "Common/ObjectAdapter.hh"
#include "VideoCompany.hh"
#include "VideoServiceProvider.hh"
#include "VideoSupplier.hh"

VideoServiceProvider *VideoSupplier::m_pServiceProvider = 0;

VideoSupplier::VideoSupplier( char *sName, CORBA::Boolean bStore) : VideoCompany( sName, "Video Stream Supplier")
{

   if ( bStore)
   {
   }
}

/*
 * @func destroys a VideoSupplier object
 *
 */
VideoSupplier::~VideoSupplier()
{
// delete offers;
}

/*
 * @func maintains a store of VideoSupplier video categories
 *
 * @parm | sCategory | the given category
 *
 * @rdesc void
 */
void VideoSupplier::addCategory( char *sCategory)
{
   StringArray *array;
   CORBA::Boolean bFound = 0;
   for(ASA_Iter itb = ASA_Iter (m_videos); itb.next (array) != 0; itb.advance ())
   {
      if ( !ACE_OS::strcmp( sCategory, (*array)[0]))
      {
         bFound= 1;
         break;
      }
   }
   if ( !bFound)
   {
      m_videos.size(m_videos.size () + 1);
      m_videos[m_videos.size () - 1].size (1);
      m_videos[m_videos.size () - 1][0] = sCategory;
   }

   //cout << "INFO: Added category" << endl;
}

/*
 * @func returns the CORBA reference of this object
 *
 * @rdesc the servant object reference
 */
CORBA::Object_var VideoSupplier::getServant()
{
   return VideoSupplier::m_pServiceProvider->getObjectAdapter()->getPOA()->servant_to_reference (this);
}

/*
 * @func IDL defined funtion that returns a list of video titles for the given
 * category
 *
 * @parm | sCategory | the given category
 *
 * @rdesc unbounded sequence of strings
 */
Server::OfferList* VideoSupplier::getOffers (const char *sCategory) throw (CORBA::SystemException)
{
   Server::OfferList *offers= new Server::OfferList();
   StringArray *array;
   char **offer;

   for (ASA_Iter itb = ASA_Iter (m_videos); itb.next (array) != 0; itb.advance ())
   {
      if ( !ACE_OS::strcmp( sCategory, (*array)[0]))
      {
         cout << "Found offers for " << (*array)[0] << endl;
         int iVideos= 0;
         offers->length (array->size () - 1);

         for(StringArray::ITERATOR itb2 = StringArray::ITERATOR (*array);
              itb2.next (offer) != 0; itb2.advance ())
         {
            if ( iVideos>0)
            {
               (*offers)[iVideos-1]= ACE_OS::strdup (*offer);
            }
            iVideos++;
         }
         break;
      }
   }

   return offers;
}

/*
 * @func Adds a video title to the object
 *
 * @parm | sCategory | the given video category
 *
 * @parm | sName | the given name of the video
 *
 * @rdesc true if logged on
 */
void VideoSupplier::addVideo( const char *sCategory, const char *sName)
{
   CORBA::Boolean bFound= 0;
   int iCount= 0;
   StringArray *array;

   for(ASA_Iter itb = ASA_Iter (m_videos); itb.next (array) != 0; itb.advance ())
   {
      if ( !ACE_OS::strcmp( sCategory, (*array)[0]))
      {
         bFound= 1;
         break;
      }
      iCount++;
   }//end for
   if ( bFound)
   {
      m_videos[iCount].size (m_videos[iCount].size () + 1);
      m_videos[iCount][m_videos[iCount].size () - 1]= (char*)sName;
   }
}

#if defined (ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION)
template class ACE_Array<char*>;
template class ACE_Array_Base<char*>;
template class ACE_Array_Iterator<char*>;
template class ACE_Array<StringArray>;
template class ACE_Array_Base<StringArray>;
template class ACE_Array_Iterator<StringArray>;
#endif /* ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION */
