#include <ace/OS.h>
#include <iostream.h>

#include "TradingClient.hh"

/* @func This program is a client application that resolves a reference to the trading service
 * and performs simple Trading Service Object look-ups based on the static properties describing
 * video-on-demand suppliers.
 */
int main ( int argc, char **argv)
{
   try
   {
   // Utilities
      Accessor utils;
   
   // initialise orb
      CORBA::ORB_var pOrb= NULL;
      try
      {
         pOrb= utils.getORB(argc, argv);
         cout << "INFO: initialised orb" << endl;
      }
      catch(CORBA::Exception& e)
      {
         cout << "ERROR: Cannot initialise orb: " << e << endl;
         return 1;
      }
      // get reference to trading service
      CORBA::Object_var pObj= NULL;
   
      try
      {
         pObj = pOrb->resolve_initial_references ("TradingService");
      }
      catch (...)
      {
         cerr << "Failed to resolve TradingService" << endl;
         return -1;
      }
   
      assert( pObj.in ());
   
   // run a query and use an iterator
      CosTrading::Lookup_var lookup;
   // get a reference to the lookup interface
      try
      {
         lookup= CosTrading::Lookup::_narrow( pObj.in ());
      }
      catch (...)
      {
         cout << "ERROR: narrowing to lookup" << endl;
         return 1;
      }
   
   // set up an empty policy
      CosTrading::PolicySeq policies;
      CosTrading::Lookup::SpecifiedProps desiredProps;
      CosTrading::PropertyNameSeq_var property_seq = new CosTrading::PropertyNameSeq ();
      property_seq->length (1);
      property_seq[(CORBA::ULong)0] = ACE_OS::strdup ("title");
      desiredProps.prop_names (property_seq.in ());
   
      CosTrading::OfferSeq_var offers;
      CosTrading::PolicyNameSeq_var policiesApplied;
      CosTrading::OfferIterator_var iterator;
   
      CORBA::String_var str= CORBA::string_dup( "");
   
   // execute query
      try
      {
         lookup->query( "IDL:video_supplier:1.0",
                        str,
                        "",
                        policies,
                        desiredProps,
                        0,
                        offers,
                        iterator,
                        policiesApplied);
   
      }
      catch( CORBA::Exception& e)
      {
         cout << "ERROR: querying: " << e << endl;
         return 1;
      }
      catch (...)
      {
         cout << "ERROR: querying" << endl;
         return 1;
      }
   
   // process results
      if ( !CORBA::is_nil( iterator.in ()))
      {
         cout << "Trader lookup for all references" << endl;
   
         CORBA::Boolean bMore;
         int iCount= 0;
         do
         {
            bMore= iterator->next_n( 10, offers);
            if ( bMore)
            {
               cout << offers->length() << " Available films." << endl;
               cout << "===================" << endl;
               for ( CORBA::ULong ulCount= 0; ulCount< offers->length(); ulCount++)
               {
   //             print the title of the films.
   
                  const char *value = 0;
   
                  try
                  {
                    cout << "Title:";
                    (*offers)[ulCount].properties[0].value >>= value;
                    CORBA::String_var copy (value);
                    cout <<  copy.in () << endl;
                  }
                  catch (const CORBA::Exception& ex)
                  {
                    ex._tao_print_exception("Printing titles failed");
                     return 1;
                  }
               }//end for
            }//end if bMore
            else
            {
               if ( iCount==0)
                  cout << "INFO: no offers found" << endl;
               else
                  cout << "INFO: no more offers found" << endl;
            }
            iCount++;
         }  while( bMore);
         iterator->destroy();
      }
      else
      {
         cout << "WARNING: no iterator returned from query" << endl;
      }
   
      cout << "Press Enter to exit..." << endl;
      int iIn= cin.get();
   
      return 0;
   }
   catch(CORBA::Exception& sysEx)
   {
      cerr << sysEx << endl;
      return 1;
   }
}
