// -*- C++ -*-
// UTF16.cpp,v 1.2 2003/12/06 15:22:13 bala Exp

#include "UTF16.h"
#include "ace/OS_Memory.h"

ACE_RCSID(UCS4_UTF16, UTF16, "UTF16.cpp,v 1.2 2003/12/06 15:22:13 bala Exp")

// ****************************************************************

TAO_UTF16::TAO_UTF16 (void)
  : tcs_size_ (2)
{

}

TAO_UTF16::~TAO_UTF16 (void)
{

}

// = Documented in $ACE_ROOT/ace/CDR_Stream.h
ACE_CDR::Boolean
TAO_UTF16::read_wchar (ACE_InputCDR &cdr, ACE_CDR::WChar &x)
{
  ACE_CDR::UShort sx = 0;
  if (ACE_static_cast (ACE_CDR::Short, this->major_version(cdr)) == 1
      && ACE_static_cast (ACE_CDR::Short, this->minor_version(cdr)) == 2)
    {
      ACE_CDR::Octet len;
      if (!this->read_1 (cdr, &len) || len < 2)
        return 0;
      
      char* buf;
      size_t align = ACE_CDR::OCTET_ALIGN;
      if (cdr.adjust (len, align, buf) == 0)
        {
          // Assume stream is big endian by default
          // i.e. swap if native byte order == little endian
          ACE_CDR::Boolean swap = ACE_CDR_BYTE_ORDER;
    
          ACE_CDR::UShort *sb = ACE_reinterpret_cast(ACE_CDR::UShort *, buf);

          if (*sb == 0xFEFF || *sb == 0xFFFE)
            {
              // 0xFEFF means stream is in native order
              swap = (*sb != 0xFEFF);
              sb++;
              len--;
            }
#if defined (ACE_DISABLE_SWAP_ON_READ)
          swap = 0;
#else
          if (!swap)
            {
              x = ACE_static_cast (ACE_CDR::WChar, *sb);
            }
          else
            {
              ACE_CDR::UShort sx;
              ACE_CDR::swap_2 (ACE_reinterpret_cast(char *, sb), ACE_reinterpret_cast(char *,&sx));
              x = ACE_static_cast (ACE_CDR::WChar, sx);
            }
          return 1;
#endif
        }
      return 0;
    }
  else
    {
      if (!this->read_2 (cdr, &sx))
      {
        return 0;
      }
    }
  x = ACE_static_cast(ACE_CDR::WChar, sx);
  return 1;
}

ACE_CDR::Boolean
TAO_UTF16::read_wstring (ACE_InputCDR &cdr,
                           ACE_CDR::WChar *&x)
{
  ACE_CDR::ULong len;
  if (!this->read_4 (cdr, &len))
    {
      return 0;
    }

  if (len > 0 && len <= cdr.length())
    {
      ACE_CDR::Boolean isOnePointTwo = 0;
      
      if (ACE_static_cast (ACE_CDR::Short, this->major_version(cdr)) == 1
          && ACE_static_cast (ACE_CDR::Short, this->minor_version(cdr)) == 2)
        {
          // Convert length from number of octets to number of wchars.
          len /= this->tcs_size_;
          isOnePointTwo = 1;
        }
      
      char* buf;
      size_t align = ACE_CDR::OCTET_ALIGN;
      if (cdr.adjust (this->tcs_size_ * len, align, buf) == 0)
        {
          // Assume stream is big endian by default
          // i.e. swap if native byte order == little endian
          ACE_CDR::Boolean swap = ACE_CDR_BYTE_ORDER;
    
          ACE_CDR::UShort *sb = ACE_reinterpret_cast(ACE_CDR::UShort *, buf);

          if (*sb == 0xFEFF || *sb == 0xFFFE)
            {
              // 0xFEFF means stream is in native order
              swap = (*sb != 0xFEFF);
              sb++;
              len--;
            }
    
          // We have to allocate here - any earlier and we may allocate over size.      
          ACE_NEW_RETURN (x,
                          ACE_CDR::WChar[(isOnePointTwo ? len + 1 : len)],
                          0);
    
#if defined (ACE_DISABLE_SWAP_ON_READ)
          swap = 0;
#else
          if (!swap)
            {
              for (size_t i = 0; i < len; i++)
                {  
                  x[i] = ACE_static_cast (ACE_CDR::WChar, sb[i]);
                }
            }
          else
            {
              for (size_t j = 0; j < len; j++)
                {
                  ACE_CDR::UShort sx;
                  ACE_CDR::swap_2 (ACE_reinterpret_cast (char*, &sb[j]), ACE_reinterpret_cast(char *,&sx));
                  x[j] = ACE_static_cast (ACE_CDR::WChar,sx);
                }
            }
#endif /* ACE_DISABLE_SWAP_ON_READ */
          if (isOnePointTwo)
            {
              x[len] = '\x00';
            }
          return 1;
        }
      return 0;
    }
  else if (len == 0)
    {
      // Convert any null strings to empty strings since empty
      // strings can cause crashes. (See bug 58.)
      ACE_NEW_RETURN (x,
                      ACE_CDR::WChar[1],
                      0);
      x[0] = '\x00';
      return 1;
    }
  x = 0;
  return 0;
}

ACE_CDR::Boolean
TAO_UTF16::write_wchar (ACE_OutputCDR &cdr,
                          ACE_CDR::WChar x)
{
  // stream writing order may not be native order if do_byte_swap set
  int writeOrder = (! ACE_CDR_BYTE_ORDER && cdr.do_byte_swap ()) ||
                   (ACE_CDR_BYTE_ORDER && ! cdr.do_byte_swap () );

  ACE_CDR::UShort sx = ACE_static_cast(ACE_CDR::Short,x);
  if (ACE_static_cast (ACE_CDR::Short, this->major_version(cdr)) == 1
      && ACE_static_cast (ACE_CDR::Short, this->minor_version(cdr)) == 2)
    {
      ACE_CDR::Octet length;
      (writeOrder ? length = this->tcs_size_ + 2 : length = this->tcs_size_);
      if (this->write_1 (cdr, &length))
        {
          char* buf;
          size_t align = ACE_CDR::OCTET_ALIGN;
          if (cdr.adjust (length, align, buf) == 0)
            {
              ACE_CDR::UShort *sb = ACE_reinterpret_cast(ACE_CDR::UShort *, buf);
              if (writeOrder)
                {
                  *sb = 0xFEFF;
                  sb++;  
                }
              
#if !defined (ACE_ENABLE_SWAP_ON_WRITE)
               *sb = ACE_static_cast (ACE_CDR::UShort, x);
#else
              if (!cdr.do_byte_swap())
                {
                  *sb = ACE_static_cast (ACE_CDR::UShort, x);;
                }
               else
                {
                  ACE_CDR::UShort sx = ACE_static_cast (ACE_CDR::UShort, x);
                  ACE_CDR::swap_2 (ACE_reinterpret_cast(char *,&sx), ACE_reinterpret_cast(char *,sb));
                }
#endif /* ACE_DISABLE_SWAP_ON_WRITE */
            }
          return 1;
        }
      else
        {
          return 0;
        }
    }
  else if (ACE_static_cast (ACE_CDR::Short, this->minor_version(cdr)) == 0)
    { // wchar is not allowed with GIOP 1.0.
      errno = EINVAL;
      return 0;
    }
  return this->write_2 (cdr, &sx);
}

ACE_CDR::Boolean
TAO_UTF16::write_wstring (ACE_OutputCDR & cdr,
                            ACE_CDR::ULong len,
                            const ACE_CDR::WChar *x)
{
  ACE_CDR::Boolean needsBOM = 0;
  
  if (len != 0)
    {
      // stream writing order may not be native order if do_byte_swap is set
      if ((! ACE_CDR_BYTE_ORDER && cdr.do_byte_swap ()) ||
            (ACE_CDR_BYTE_ORDER && ! cdr.do_byte_swap ()))
        {
          // We are going to encode a BOM.
          needsBOM = 1;
          // Increase the length to accomodate it
          len++;
        }
    }
  
  ACE_CDR::ULong l = 0;
  
  if (ACE_static_cast (ACE_CDR::Short, this->major_version(cdr)) == 1
      && ACE_static_cast (ACE_CDR::Short, this->minor_version(cdr)) == 2)
    {
      // encoded 'length' is the number of octets
      l = len * this->tcs_size_;
    }
  else
    {      
      // We're going to encode the null terminator
      len++;
      // encoded 'length' is the number of wchars (incl null)
      l = len;
    }
  
  // encode the 'length'
  if (this->write_4 (cdr, &l))
    {
      char* buf;
      size_t align = ACE_CDR::OCTET_ALIGN;
      if (cdr.adjust (this->tcs_size_ * len, align, buf) == 0)
        {
          ACE_CDR::UShort *sb = ACE_reinterpret_cast(ACE_CDR::UShort *, buf);
          if (needsBOM)
            {
              *sb = 0xFEFF;
              sb++;  
            }
          for (size_t i = 0; i < len; i++)
            {
#if !defined (ACE_ENABLE_SWAP_ON_WRITE)
              sb[i] = ACE_static_cast (ACE_CDR::UShort, x[i]);
#else
              if (!cdr.do_byte_swap())
                {
                  sb[i] = ACE_static_cast (ACE_CDR::UShort, x[i]);;
                }
              else
                {
                  ACE_CDR::UShort sx = ACE_static_cast (ACE_CDR::UShort, x[i]);
                  ACE_CDR::swap_2 (ACE_reinterpret_cast(char *,&sx), ACE_reinterpret_cast(char *,&sb[i]));
                }
#endif /* ACE_DISABLE_SWAP_ON_WRITE */
            }
        }
      return 1;
    }
      
  return 0;
}

ACE_CDR::Boolean 
TAO_UTF16::read_wchar_array (ACE_InputCDR& cdr,
                             ACE_CDR::WChar* x,
                             ACE_CDR::ULong length)
{
  if (length == 0)
    return 1;
  for (size_t i = 0; i < length; i++)
    {
      if (!read_wchar(cdr, x[i]))
        {
          return 0;
        }
    }
  return 1;
}

ACE_CDR::Boolean 
TAO_UTF16::write_wchar_array (ACE_OutputCDR& cdr,
                              const ACE_CDR::WChar* x,
                              ACE_CDR::ULong length)
{
  if (length == 0)
    return 1;
  for (size_t i = 0; i < length; i++)
    {
      if (!write_wchar(cdr, x[i]))
        {
          return 0;
        }
    }
  return 1;
}
